@@ -4,6 +4,10 @@
  * IO engine that doesn't do any real IO transfers, it just pretends to.
  * The main purpose is to test fio itself.
  *
+ * It also can act as external C++ engine - compiled with:
+ *
+ * g++ -O2 -g -shared -rdynamic -fPIC -o null.so null.c -DFIO_EXTERNAL_ENGINE
+ *
  */
 #include <stdio.h>
 #include <stdlib.h>
@@ -21,15 +25,16 @@ struct null_data {
 
 static struct io_u *fio_null_event(struct thread_data *td, int event)
 {
-	struct null_data *nd = td->io_ops->data;
+	struct null_data *nd = (struct null_data *) td->io_ops->data;
 
 	return nd->io_us[event];
 }
 
-static int fio_null_getevents(struct thread_data *td, int min_events,
-			      int fio_unused max, struct timespec fio_unused *t)
+static int fio_null_getevents(struct thread_data *td, unsigned int min_events,
+			      unsigned int fio_unused max,
+			      struct timespec fio_unused *t)
 {
-	struct null_data *nd = td->io_ops->data;
+	struct null_data *nd = (struct null_data *) td->io_ops->data;
 	int ret = 0;
 	
 	if (min_events) {
@@ -42,9 +47,12 @@ static int fio_null_getevents(struct thread_data *td, int min_events,
 
 static int fio_null_commit(struct thread_data *td)
 {
-	struct null_data *nd = td->io_ops->data;
+	struct null_data *nd = (struct null_data *) td->io_ops->data;
 
 	if (!nd->events) {
+#ifndef FIO_EXTERNAL_ENGINE
+		io_u_mark_submit(td, nd->queued);
+#endif
 		nd->events = nd->queued;
 		nd->queued = 0;
 	}
@@ -52,9 +60,9 @@ static int fio_null_commit(struct thread_data *td)
 	return 0;
 }
 
-static int fio_null_queue(struct thread_data fio_unused *td, struct io_u *io_u)
+static int fio_null_queue(struct thread_data *td, struct io_u *io_u)
 {
-	struct null_data *nd = td->io_ops->data;
+	struct null_data *nd = (struct null_data *) td->io_ops->data;
 
 	fio_ro_check(td, io_u);
 
@@ -75,24 +83,23 @@ static int fio_null_open(struct thread_data fio_unused *td,
 
 static void fio_null_cleanup(struct thread_data *td)
 {
-	struct null_data *nd = td->io_ops->data;
+	struct null_data *nd = (struct null_data *) td->io_ops->data;
 
 	if (nd) {
 		if (nd->io_us)
 			free(nd->io_us);
 		free(nd);
-		td->io_ops->data = NULL;
 	}
 }
 
 static int fio_null_init(struct thread_data *td)
 {
-	struct null_data *nd = malloc(sizeof(*nd));
+	struct null_data *nd = (struct null_data *) malloc(sizeof(*nd));
 
 	memset(nd, 0, sizeof(*nd));
 
 	if (td->o.iodepth != 1) {
-		nd->io_us = malloc(td->o.iodepth * sizeof(struct io_u *));
+		nd->io_us = (struct io_u **) malloc(td->o.iodepth * sizeof(struct io_u *));
 		memset(nd->io_us, 0, td->o.iodepth * sizeof(struct io_u *));
 	} else
 		td->io_ops->flags |= FIO_SYNCIO;
@@ -101,6 +108,7 @@ static int fio_null_init(struct thread_data *td)
 	return 0;
 }
 
+#ifndef __cplusplus
 static struct ioengine_ops ioengine = {
 	.name		= "null",
 	.version	= FIO_IOOPS_VERSION,
@@ -123,3 +131,30 @@ static void fio_exit fio_null_unregister(void)
 {
 	unregister_ioengine(&ioengine);
 }
+
+#else
+
+#ifdef FIO_EXTERNAL_ENGINE
+extern "C" {
+void get_ioengine(struct ioengine_ops **ioengine_ptr)
+{
+	struct ioengine_ops *ioengine;
+
+	*ioengine_ptr = (struct ioengine_ops *) malloc(sizeof(struct ioengine_ops));
+	ioengine = *ioengine_ptr;
+
+	strcpy(ioengine->name, "cpp_null");
+	ioengine->version        = FIO_IOOPS_VERSION;
+	ioengine->queue          = fio_null_queue;
+	ioengine->commit         = fio_null_commit;
+	ioengine->getevents      = fio_null_getevents;
+	ioengine->event          = fio_null_event;
+	ioengine->init           = fio_null_init;
+	ioengine->cleanup        = fio_null_cleanup;
+	ioengine->open_file      = fio_null_open;
+	ioengine->flags	         = FIO_DISKLESSIO;
+}
+}
+#endif /* FIO_EXTERNAL_ENGINE */
+
+#endif /* __cplusplus */
